@@ -64,6 +64,16 @@ class Tools:
             else:
                 os.mkdir(dir)
 
+    def mk_class_dir(self,class_name,result_root_this_script):
+        this_class_arr = join(result_root_this_script,f'arr/{class_name}/')
+        this_class_tif = join(result_root_this_script,f'tif/{class_name}/')
+        this_class_png = join(result_root_this_script,f'png/{class_name}/')
+        self.mk_dir(this_class_arr, force=True)
+        self.mk_dir(this_class_tif, force=True)
+        self.mk_dir(this_class_png, force=True)
+
+        return this_class_arr,this_class_tif,this_class_png
+
     def path_join(self,*args):
         path = os.path.join(*args)
         return path
@@ -828,15 +838,19 @@ class Tools:
                 raise UserWarning(f'Error grow_season:{grow_season}')
         monthly_vals = np.array(monthly_vals)
         monthly_vals_reshape = np.reshape(monthly_vals,(-1,12))
+        monthly_vals_reshape_T = monthly_vals_reshape.T
+        monthly_vals_reshape_T_gs = Tools().pick_vals_from_1darray(monthly_vals_reshape_T,grow_season)
+        monthly_vals_reshape_gs = monthly_vals_reshape_T_gs.T
         annual_val_list = []
-        for one_year_vals in monthly_vals_reshape:
-            one_year_vals_pick = Tools().pick_vals_from_1darray(one_year_vals,grow_season)
+        for one_year_vals in monthly_vals_reshape_gs:
             if method == 'mean':
-                annual_val = np.nanmean(one_year_vals_pick)
+                annual_val = np.nanmean(one_year_vals)
             elif method == 'max':
-                annual_val = np.nanmax(one_year_vals_pick)
+                annual_val = np.nanmax(one_year_vals)
             elif method == 'min':
-                annual_val = np.nanmin(one_year_vals_pick)
+                annual_val = np.nanmin(one_year_vals)
+            elif method == 'array':
+                annual_val = np.array(one_year_vals)
             else:
                 raise UserWarning(f'method:{method} error')
             annual_val_list.append(annual_val)
@@ -844,6 +858,21 @@ class Tools:
 
         return annual_val_list
 
+    def monthly_vals_to_date_dic(self, monthly_val, start_year, end_year):
+        date_list = []
+        for y in range(start_year, end_year + 1):
+            for m in range(1, 13):
+                date = f'{y}{m:02d}'
+                date_list.append(date)
+        len_vals = len(monthly_val)
+        len_date_list = len(date_list)
+        if not len_vals == len_date_list:
+            raise UserWarning('Date list is not matching value list')
+        dic = dict(zip(date_list, monthly_val))
+
+        return dic
+
+
 class SMOOTH:
     '''
     一些平滑算法
@@ -962,7 +991,13 @@ class SMOOTH:
 
         y = np.convolve(w / w.sum(), s, mode='valid')
         # return y
-        return y[(window_len // 2 - 1):-(window_len // 2)]
+        y_interpolated = y[(window_len // 2 - 1):-(window_len // 2)]
+        if len(x) == len(y_interpolated):
+            return y_interpolated
+        elif len(x) < len(y_interpolated):
+            return y_interpolated[1:]
+        else:
+            raise UserWarning('Need debug...')
 
     def smooth(self, x):
         # 后窗滤波
@@ -1944,13 +1979,13 @@ class KDE_plot:
         if is_equal:
             plt.axis('equal')
         if plot_fit_line:
-            a, b, r = self.linefit(val1, val2)
+            a, b, r, p = self.linefit(val1, val2)
             if is_plot_1_1_line:
                 plt.plot([np.min([val1, val2]), np.max([val1, val2])], [np.min([val1, val2]), np.max([val1, val2])],
                          '--', c='k')
-            self.plot_fit_line(a, b, r, val1, line_color=fit_line_c)
+            self.plot_fit_line(a, b, r, p, val1, line_color=fit_line_c)
             # plt.legend()
-            return a, b, r
+            return a, b, r, p
 
     def cmap_with_transparency(self, cmap, min_alpha=0., max_alpha=0.5):
         ncolors = 256
@@ -1982,7 +2017,8 @@ class Pre_Process:
 
         pass
 
-    def data_transform(self, fdir, outdir):
+    def data_transform(self, fdir, outdir, n=10000):
+        n = int(n)
         # 不可并行，内存不足
         Tools().mk_dir(outdir)
         # 将空间图转换为数组
@@ -2039,13 +2075,14 @@ class Pre_Process:
             arr = void_dic[key]
             arr = np.array(arr)
             temp_dic[key] = arr
-            if flag % 10000 == 0:
+            if flag % n == 0:
                 # print('\nsaving %02d' % (flag / 10000)+'\n')
-                np.save(outdir + '/per_pix_dic_%03d' % (flag / 10000), temp_dic)
+                np.save(outdir + '/per_pix_dic_%03d' % (flag / n), temp_dic)
                 temp_dic = {}
         np.save(outdir + '/per_pix_dic_%03d' % 0, temp_dic)
 
-    def data_transform_with_date_list(self, fdir, outdir, date_list):
+    def data_transform_with_date_list(self, fdir, outdir, date_list, n=10000):
+        n = int(n)
         # 不可并行，内存不足
         Tools().mk_dir(outdir)
         outdir = outdir + '/'
@@ -2097,9 +2134,9 @@ class Pre_Process:
             arr = void_dic[key]
             arr = np.array(arr)
             temp_dic[key] = arr
-            if flag % 10000 == 0:
+            if flag % n == 0:
                 # print('\nsaving %02d' % (flag / 10000)+'\n')
-                np.save(outdir + 'per_pix_dic_%03d' % (flag / 10000), temp_dic)
+                np.save(outdir + 'per_pix_dic_%03d' % (flag / n), temp_dic)
                 temp_dic = {}
         np.save(outdir + 'per_pix_dic_%03d' % 0, temp_dic)
 
@@ -2276,7 +2313,7 @@ class Pre_Process:
             DIC_and_TIF(tif_template=tif_template).arr_to_tif(arr,outf)
 
 
-class Plot_line:
+class Plot:
     def __init__(self):
 
         pass
@@ -2314,6 +2351,29 @@ class Plot_line:
                              color=c, edgecolor=None, **kwargs)
         pass
 
+    def plot_line_with_error_bar(self,x, y, yerr,c=None,alpha=0.2, **kwargs):
+
+        x = np.array(x)
+        y = np.array(y)
+        yerr = np.array(yerr)
+        y1 = y + yerr
+        y2 = y - yerr
+        plt.fill_between(x, y1, y2,zorder=-99,
+                             color=c, edgecolor=None,alpha=0.2, **kwargs)
+
+    def plot_hist_smooth(self,arr,interpolate_window=5,**kwargs):
+        weights = np.ones_like(arr) / float(len(arr))
+
+        n1, x1, patch = plt.hist(arr,weights=weights,**kwargs)
+        density1 = stats.gaussian_kde(arr)
+        y1 = density1(x1)
+        coe = max(n1) / max(y1)
+        y1 = y1 * coe
+        y1 = SMOOTH().smooth_convolve(y1, interpolate_window)
+        return x1,y1
+
+        pass
+
 
 class ToRaster:
     def __init__(self):
